@@ -16,11 +16,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #
-# CasualServices: Search and share your favourite places from 11870.com
-# Version 0.3
+# CasualServices: Search and share your favourite places
+# Version 0.4
 #
 
-VERSION = "0.3"
+VERSION = "0.4"
 
 import gtk
 import hildon
@@ -33,6 +33,7 @@ import pango
 import gettext
 import osso
 import settings
+import cgi
 
 try:
     import location
@@ -42,6 +43,8 @@ except:
 from services import once
 from services import glocal
 
+from portrait import FremantleRotation
+
 ossoc = osso.Context("org.maemo.cservices", VERSION, False)
 
 #Some useful long functions used in the code, we use smaller alias
@@ -96,7 +99,7 @@ if DEBUG:
     logging.getLogger('').addHandler(console)
     logger.debug("Starting log")
 
-
+#Define some global list for services details
 titles = []
 phones = []
 urls = []
@@ -109,18 +112,26 @@ areas = []
 medialinks = []
 listimg = []
 
+latitudes = []
+longitudes = []
+
+#Some of this map sources doesn't work
+#mapsources = {osmgpsmap.SOURCE_OPENSTREETMAP: 'Openstreemap',
+#osmgpsmap.SOURCE_GOOGLE_STREET: 'Google Street',
+#osmgpsmap.SOURCE_GOOGLE_HYBRID: 'Google Hybrid',
+#osmgpsmap.SOURCE_GOOGLE_SATELLITE: 'Google Satellite',
+#osmgpsmap.SOURCE_MAPS_FOR_FREE: 'Maps for free',
+#osmgpsmap.SOURCE_OPENAERIALMAP: 'Open Aerial map',
+#osmgpsmap.SOURCE_VIRTUAL_EARTH_HYBRID: 'Virtual Earth Hybrid',
+#osmgpsmap.SOURCE_VIRTUAL_EARTH_SATELLITE: 'Virtual Earth Satellite',
+#osmgpsmap.SOURCE_VIRTUAL_EARTH_STREET: 'Virtual Earth Street',
+#osmgpsmap.SOURCE_YAHOO_HYBRID: 'Yahoo Hybrid',
+#osmgpsmap.SOURCE_YAHOO_SATELLITE: 'Yahoo Satellite',
+#osmgpsmap.SOURCE_YAHOO_STREET: 'Yahoo Street'}
+
 mapsources = {osmgpsmap.SOURCE_OPENSTREETMAP: 'Openstreemap',
 osmgpsmap.SOURCE_GOOGLE_STREET: 'Google Street',
-osmgpsmap.SOURCE_GOOGLE_HYBRID: 'Google Hybrid',
-osmgpsmap.SOURCE_GOOGLE_SATELLITE: 'Google Satellite',
-osmgpsmap.SOURCE_MAPS_FOR_FREE: 'Maps for free',
-osmgpsmap.SOURCE_OPENAERIALMAP: 'Open Aerial map',
-osmgpsmap.SOURCE_VIRTUAL_EARTH_HYBRID: 'Virtual Earth Hybrid',
-osmgpsmap.SOURCE_VIRTUAL_EARTH_SATELLITE: 'Virtual Earth Satellite',
-osmgpsmap.SOURCE_VIRTUAL_EARTH_STREET: 'Virtual Earth Street',
-osmgpsmap.SOURCE_YAHOO_HYBRID: 'Yahoo Hybrid',
-osmgpsmap.SOURCE_YAHOO_SATELLITE: 'Yahoo Satellite',
-osmgpsmap.SOURCE_YAHOO_STREET: 'Yahoo Street'}
+osmgpsmap.SOURCE_GOOGLE_SATELLITE: 'Google Satellite'}
 
 defaultimg = imgdir + 'defaultimg.png'
 defaultpbuf = gtk.gdk.pixbuf_new_from_file(defaultimg)
@@ -196,6 +207,7 @@ class MainWindow:
         self.window.connect("destroy", gtk.main_quit)
         self.program.add_window(self.window)
 
+        self.rotation = FremantleRotation('cservices', None, VERSION, 0)
         #Variable for pagination in searches, the first page is 1 obviously
         self.page = '1'
         self.radius = '10'
@@ -217,6 +229,7 @@ class MainWindow:
         #Hide the buttons to change the page in the searches
         self.backbtn.hide()
         self.forwbtn.hide()
+        self.allmapbtn.hide()
 
     def initial_screen(self):
         """Return a gtk.VBox with the UI widgets for the initial screen.
@@ -298,28 +311,35 @@ class MainWindow:
 
         #Search button
         button = hildon.Button(fhsize, horbtn)
-        img = gtk.Image()
-        img.set_from_file('/usr/share/icons/hicolor/48x48/hildon/general_search.png')
+        img = gtk.image_new_from_icon_name("general_search",
+                                            gtk.ICON_SIZE_LARGE_TOOLBAR)
         button.set_image(img)
         button.connect("clicked", self.search_services)
         hbox.pack_start(button, False, False, 5)
 
         #Previous page button (for searches)
         self.backbtn = hildon.Button(fhsize, horbtn)
-        img = gtk.Image()
-        img.set_from_file('/usr/share/icons/hicolor/48x48/hildon/general_back.png')
+        img = gtk.image_new_from_icon_name("general_back",
+                                            gtk.ICON_SIZE_LARGE_TOOLBAR)
         self.backbtn.set_image(img)
         self.backbtn.connect("clicked", self.search_prev_page)
         hbox.pack_start(self.backbtn, False, False, 5)
 
         #Next page button (for searches)
         self.forwbtn = hildon.Button(fhsize, horbtn)
-        img = gtk.Image()
-        img.set_from_file('/usr/share/icons/hicolor/48x48/hildon/general_forward.png')
+        img = gtk.image_new_from_icon_name("general_forward",
+                                            gtk.ICON_SIZE_LARGE_TOOLBAR)
         self.forwbtn.set_image(img)
         self.forwbtn.connect("clicked", self.search_next_page)
         hbox.pack_start(self.forwbtn, False, False, 5)
 
+        self.allmapbtn = hildon.Button(fhsize, horbtn)
+        img = gtk.image_new_from_icon_name("general_map",
+                                            gtk.ICON_SIZE_LARGE_TOOLBAR)
+        self.allmapbtn.set_image(img)
+        self.allmapbtn.connect("clicked", self.show_all_in_map)
+        hbox.pack_start(self.allmapbtn, False, False, 5)
+
         return hbox
 
     def make_menu(self):
@@ -347,6 +367,10 @@ class MainWindow:
         self.nearbtn.set_active(False)
         menu.append(self.nearbtn)
 
+        button = gtk.Button(_("Your services at 11870.com"))
+        button.connect("clicked", self.get_user_sites)
+        menu.append(button)
+
         button = gtk.Button(_("Settings"))
         button.connect("clicked", self.show_settings_dlg)
         menu.append(button)
@@ -355,13 +379,10 @@ class MainWindow:
         button.connect("clicked", About)
         menu.append(button)
 
-        button = gtk.Button(_("Your services"))
-        button.connect("clicked", self.get_user_sites)
-        menu.append(button)
 
-        button = gtk.Button(_("Create new service"))
-        button.connect("clicked", self.create_service)
-        menu.append(button)
+        #button = gtk.Button(_("Create new service"))
+        #button.connect("clicked", self.create_service)
+        #menu.append(button)
 
         menu.show_all()
         return menu
@@ -539,15 +560,16 @@ class MainWindow:
             #If there are search results, add it to the gtk.liststore
             for i in range(len(searchlist[0])):
                 liter = lstore.append()
-                txtformatted = lstore._markup_template % (searchlist[0][i],
-                                searchlist[1][i], searchlist[2][i],
-                                searchlist[3][i])
+                txtformatted = lstore._markup_template % (cgi.escape(searchlist[0][i]),
+                                cgi.escape(searchlist[1][i]),
+                                cgi.escape(searchlist[2][i]),
+                                cgi.escape(searchlist[3][i]))
 
-                lstore.set(liter, 0, self.set_pix(searchlist[4][i]), 1, txtformatted)
+                lstore.set(liter, 0, self.set_pix(searchlist[4][i], False), 1, txtformatted)
 
         return lstore
 
-    def set_pix(self, urlimg):
+    def set_pix(self, urlimg, infoscreen):
         """Return a gtk.gdk.Pixbuf with the image from the URL
 
         @type   urlimg: string
@@ -570,7 +592,10 @@ class MainWindow:
 
             return pbuf
         except:
-            return defaultpbuf
+            if infoscreen:
+                return None
+            else:
+                return defaultpbuf
 
     def add_columns_to_searchlist(self, treeview):
         """Add gtk.TreeviewColumns to the gtk.TreeView for searchs
@@ -594,10 +619,6 @@ class MainWindow:
         treeview.append_column(column)
 
     def selected_service(self, widget, path, column):
-        winprogind(self.window, 1)
-        thread.start_new(self._do_show_service_info, (widget, path, column))
-
-    def _do_show_service_info(self, widget, path, column):
         """Show a new window with the information from a selected service
 
         @type   widget: gtk.Widget
@@ -608,14 +629,30 @@ class MainWindow:
         @param  column: selected column from the treeview
 
         """
-        win = hildon.StackableWindow()
-        win.set_title(_("Information"))
+        self.serwin = hildon.StackableWindow()
+        self.serwin.get_screen().connect("size-changed", self.orientation_changed)
+        self.serwin.set_title(_("Information"))
+
+        menu = hildon.AppMenu()
+        button = gtk.Button(_("Add to Contacts"))
+        button.connect("clicked", self.add_to_contacts, path)
+        menu.append(button)
+        menu.show_all()
+        self.serwin.set_app_menu(menu)
 
         vbox = gtk.VBox()
 
+        #TODO: show an image with the site of the result??
+        #btn = gtk.Button()
+        #img = gtk.image_new_from_file(imgdir + '11870.png')
+        #btn.set_image(img)
+        #btn.set_relief(gtk.RELIEF_NONE)
+        #btn.set_alignment(0.00, 0.5)
+        #vbox.pack_start(btn, True, True, 0)
+
         label = gtk.Label()
         label.set_alignment(0.00, 0.5)
-        label.set_markup('<b>%s:</b> %s' % (_("Title"), titles[path[0]]))
+        label.set_markup('<b>%s:</b> %s' % (_("Title"), cgi.escape(titles[path[0]])))
         label.set_line_wrap(gtk.WRAP_WORD)
         vbox.pack_start(label, True, True, 0)
 
@@ -631,15 +668,9 @@ class MainWindow:
             else:
                 summary = summaries[path[0]]
 
-            label.set_markup("<b>%s:</b> %s" % (_("Summary"), summary))
+            label.set_markup("<b>%s:</b> %s" % (_("Summary"), cgi.escape(summary)))
             vbox.pack_start(label, True, True, 0)
 
-        #label = gtk.Label()
-        #label.set_alignment(0.00, 0.5)
-        #label.set_markup('<b>%s:</b> %s' % (_("Phone"), phones[path[0]]))
-        #label.set_line_wrap(gtk.WRAP_WORD)
-        #vbox.pack_start(label, True, True, 0)
-
         if phones[path[0]] != _('Not available'):
             button = hildon.Button(fhsize, verbtn)
             button.set_title(_('Phone'))
@@ -647,28 +678,27 @@ class MainWindow:
             button.connect("clicked", self.make_phonecall, phones[path[0]])
             vbox.pack_start(button, False, False, 0)
 
-
         label = gtk.Label()
         label.set_alignment(0.00, 0.5)
-        label.set_markup('<b>%s:</b> %s' % (_("Address"), addresses[path[0]]))
+        label.set_markup('<b>%s:</b> %s' % (_("Address"), cgi.escape(addresses[path[0]])))
         label.set_line_wrap(gtk.WRAP_WORD)
         vbox.pack_start(label, True, True, 0)
 
         label = gtk.Label()
         label.set_alignment(0.00, 0.5)
-        label.set_markup('<b>%s:</b> %s' % (_("Locality"), localities[path[0]]))
+        label.set_markup('<b>%s:</b> %s' % (_("Locality"), cgi.escape(localities[path[0]])))
         label.set_line_wrap(gtk.WRAP_WORD)
         vbox.pack_start(label, True, True, 0)
 
         label = gtk.Label()
         label.set_alignment(0.00, 0.5)
-        label.set_markup('<b>%s:</b> %s' % (_("Area"), areas[path[0]]))
+        label.set_markup('<b>%s:</b> %s' % (_("Area"), cgi.escape(areas[path[0]])))
         label.set_line_wrap(gtk.WRAP_WORD)
         vbox.pack_start(label, True, True, 0)
 
         label = gtk.Label()
         label.set_alignment(0.00, 0.5)
-        label.set_markup('<b>%s:</b> %s' % (_("Country"), countries[path[0]]))
+        label.set_markup('<b>%s:</b> %s' % (_("Country"), cgi.escape(countries[path[0]])))
         label.set_line_wrap(gtk.WRAP_WORD)
         vbox.pack_start(label, True, True, 0)
 
@@ -680,72 +710,205 @@ class MainWindow:
             vbox.pack_start(button, False, False, 0)
 
         if medialinks[path[0]]:
-            hbox = gtk.HBox()
-            for i in range(len(medialinks[path[0]])):
-                img = gtk.Image()
-                pix = self.set_pix(medialinks[path[0]][i])
-                img.set_from_pixbuf(pix)
-                hbox.pack_start(img, True, True, 0)
-            vbox.pack_start(hbox, True, True, 0)
+            self.mediabox = gtk.HBox()
+            vbox.pack_start(self.mediabox, True, True, 0)
 
-        hbox = gtk.HBox()
-        parea = hildon.PannableArea()
-        parea.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)
-        parea.set_size_request(540, -1)
-        parea.add_with_viewport(vbox)
-        hbox.pack_start(parea, False, False, 0)
+        self.serparea = hildon.PannableArea()
+        self.serparea.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)
+        self.serparea.add_with_viewport(vbox)
 
-        vbox = gtk.VBox()
+        self.mapbox = gtk.VBox()
+        self.map = osmgpsmap.GpsMap(repo_uri = \
+                    osmgpsmap.source_get_repo_uri(int(self.mapsource)), \
+                    tile_cache = HOME + '/MyDocs/.maps')
 
-        button = hildon.Button(fhsize, horbtn)
-        button.connect("clicked", self.show_your_position_on_map)
-        button.set_label(_("Show your position"))
-        vbox.pack_start(button, False, False, 0)
-
-        self.map = osmgpsmap.GpsMap(repo_uri=osmgpsmap.source_get_repo_uri(int(self.mapsource)))
         lat, lon = gmlpos[path[0]].split(' ')
-
-        vbox.pack_start(self.map, True, True, 0)
-
+        self.mapbox.pack_start(self.map, True, True, 0)
 
         hbox1 = gtk.HBox()
         button = hildon.Button(fhsize, horbtn)
-        button.connect("clicked", self.zoomin_map)
+        button.connect("clicked", self.zoomin_map, self.map)
         img = gtk.image_new_from_icon_name("pdf_zoomin",
                                             gtk.ICON_SIZE_LARGE_TOOLBAR)
         button.set_image(img)
         hbox1.pack_start(button, True, True, 0)
 
         button = hildon.Button(fhsize, horbtn)
-        button.connect("clicked", self.center_map_on_service, lat, lon)
+        button.connect("clicked", self.center_map_on_service, lat, lon, self.map)
         img = gtk.image_new_from_icon_name("sketch_circle",
                                             gtk.ICON_SIZE_LARGE_TOOLBAR)
         button.set_image(img)
         hbox1.pack_start(button, True, True, 0)
 
         button = hildon.Button(fhsize, horbtn)
-        button.connect("clicked", self.zoomout_map)
+        button.connect("clicked", self.zoomout_map, self.map)
         img = gtk.image_new_from_icon_name("pdf_zoomout",
                                             gtk.ICON_SIZE_LARGE_TOOLBAR)
         button.set_image(img)
         hbox1.pack_start(button, True, True, 0)
 
-        vbox.pack_start(hbox1, False, False, 0)
+        button = hildon.Button(fhsize, horbtn)
+        button.connect("clicked", self.show_your_position_on_map, self.map)
+        img = gtk.image_new_from_icon_name("gps_location",
+                                            gtk.ICON_SIZE_LARGE_TOOLBAR)
+        button.set_image(img)
+        hbox1.pack_start(button, True, True, 0)
 
-        hbox.pack_start(vbox, True, True, 0)
+        self.mapbox.pack_start(hbox1, False, False, 0)
 
-        win.add(hbox)
-        win.show_all()
+        self.horbox = gtk.HBox()
+        self.verbox = gtk.VBox()
+
+        if self.is_portrait():
+            self.serparea.set_size_request(-1, 480)
+            self.verbox.pack_start(self.serparea, False, False, 0)
+            self.verbox.pack_start(self.mapbox, True, True, 0)
+            self.serwin.add(self.verbox)
+        else:
+            self.serparea.set_size_request(480, -1)
+            self.horbox.pack_start(self.serparea, False, False, 0)
+            self.horbox.pack_start(self.mapbox, True, True, 0)
+            self.serwin.add(self.horbox)
+
+        self.serwin.show_all()
         try:
             #Centering the map in the service position
             self.map.set_mapcenter (float(lat), float(lon), 13)
         except ValueError:
             pass
 
-        winprogind(self.window, 0)
+
         self.add_pixmap_to_map(lat, lon)
 
+        if medialinks[path[0]]:
+            self.show_images(self.serwin, path)
+
+    def show_images(self, win, path):
+        """Start a thread to call the function to show images from a URL
+
+        @type   win: gtk.Window
+        @param  win: gtk.Window where show the animation icon
+        @type   path: number
+        @param  path: path from the selected row of the treeview
+
+        """
+        winprogind(win, 1)
+        thread.start_new(self._do_show_images, (win, path))
+
+    def _do_show_images(self, win, path):
+        """Show images from URL in the information screen
+
+        @type   win: gtk.Window
+        @param  win: gtk.Window where show the animation icon
+        @type   path: number
+        @param  path: path from the selected row of the treeview
+
+        """
+        for i in range(len(medialinks[path[0]])):
+            img = gtk.Image()
+            pix = self.set_pix(medialinks[path[0]][i], True)
+            img.set_from_pixbuf(pix)
+            self.mediabox.pack_start(img, True, True, 0)
+
+        self.mediabox.show_all()
+        winprogind(win, 0)
+
+    def add_to_contacts(self, widget, path):
+        """Show images from URL in the information screen
+
+        @type   widget: gtk.Widget
+        @param  widget: gtk.Widget that emits the signal
+        @type   path: number
+        @param  path: path from the selected row of the treeview
+
+        """
+        import agenda
+        ag = agenda.Agenda()
+        #data = [name, address, phone, locality, region, country, url, img]
+        #print titles[path[0]]       #sidreria yumay
+        #print addresses[path[0]]    #Rafael Suárez, 7
+        #print phones[path[0]]       #+34 985 570 826
+        #print localities[path[0]]   #Avilés
+        #print areas[path[0]]        #Asturias
+        #print countries[path[0]]    #España
+        #print urls[path[0]]         #http://www.sidreriayumay.com/
+        #print medialinks[path[0]]   #[]
+
+        data = [titles[path[0]], addresses[path[0]],
+                phones[path[0]].replace(' ', ''), localities[path[0]],
+                areas[path[0]], countries[path[0]],
+                urls[path[0]], medialinks[path[0]]]
+        result = ag.add_contact(data)
+
+        if result:
+            self.show_info_banner(widget, _("Service added to Contacts"))
+
+    def is_portrait(self):
+        """Check if the window is in portrait
+
+        @rtype: boolean
+        @returns: True if in portrait, False if in landscape.
+
+        """
+        width = gtk.gdk.screen_width()
+        height = gtk.gdk.screen_height()
+        if width > height:
+            return False
+        else:
+            return True
+
+    def orientation_changed(self, screen):
+        """Reorganize widgets in info screen when changing from portrait
+        to landscape or viceversa.
+
+        @type   screen: gtk.gdk.Screen
+        @param  screen: gtk.gdk.Screen that emits the signal
+
+        """
+
+        if not self.map.get_parent_window():
+            print 'not in the right screen, doing nothing'
+            return
+
+        try:
+            if self.is_portrait():
+                self.serparea.set_size_request(-1, 480)
+
+                self.horbox.remove(self.serparea)
+                self.horbox.remove(self.mapbox)
+                self.serwin.remove(self.horbox)
+
+                self.verbox.pack_start(self.serparea, False, False, 0)
+                self.verbox.pack_start(self.mapbox, True, True, 0)
+
+                self.serwin.add(self.verbox)
+                self.serwin.show_all()
+            else:
+                self.serparea.set_size_request(480, -1)
+
+                self.verbox.remove(self.serparea)
+                self.verbox.remove(self.mapbox)
+                self.serwin.remove(self.verbox)
+
+                self.horbox.pack_start(self.serparea, False, False, 0)
+                self.horbox.pack_start(self.mapbox, True, True, 0)
+
+                self.serwin.add(self.horbox)
+                self.serwin.show_all()
+        except:
+            pass
+
+
     def add_pixmap_to_map(self, lat, lon):
+        """Add pixbuf to map in a defined position
+
+        @type   lat: text
+        @param  lat: Geographic latitude.
+        @type   lon: text
+        @param  lon: Geographic longitude.
+
+        """
+
         try:
             pixbuf = gtk.gdk.pixbuf_new_from_file(imgdir + 'place.png')
             self.map.add_image(float(lat), float(lon), pixbuf)
@@ -769,7 +932,7 @@ class MainWindow:
         iface = dbus.Interface(proxy, 'com.nokia.osso_browser')
         iface.open_new_window(authurl)
 
-    def show_your_position_on_map(self, widget):
+    def show_your_position_on_map(self, widget, themap):
         """Show the position of the GPS in the map
 
         @type   widget: gtk.Widget
@@ -784,28 +947,28 @@ class MainWindow:
         print lat, lon
 
         #Draw our position in the map
-        self.map.draw_gps(float(lat), float(lon), 1.0)
+        themap.draw_gps(float(lat), float(lon), 1.0)
 
 
-    def zoomin_map(self, widget):
+    def zoomin_map(self, widget, themap):
         """Zoom in the map
 
         @type   widget: gtk.Widget
         @param  widget: gtk.Widget that emits the signal
 
         """
-        self.map.set_zoom(self.map.props.zoom + 1)
+        themap.set_zoom(themap.props.zoom + 1)
 
-    def zoomout_map(self, widget):
+    def zoomout_map(self, widget, themap):
         """Zoom out the map
 
         @type   widget: gtk.Widget
         @param  widget: gtk.Widget that emits the signal
 
         """
-        self.map.set_zoom(self.map.props.zoom - 1)
+        themap.set_zoom(themap.props.zoom - 1)
 
-    def center_map_on_service(self, widget, lat, lon):
+    def center_map_on_service(self, widget, lat, lon, themap):
         """Center the map where the service is located
 
         @type   widget: gtk.Widget
@@ -816,7 +979,7 @@ class MainWindow:
         @param  lon: Geographic longitude of the service
 
         """
-        self.map.set_mapcenter (float(lat), float(lon), 13)
+        themap.set_mapcenter (float(lat), float(lon), 13)
 
     def search_services(self, widget):
         """Start a new thread to search for services
@@ -851,22 +1014,27 @@ class MainWindow:
 
             #if option to search "near your position" is active
             if self.nearbtn.get_active():
-                glocalresults = {}
+                #glocalresults = {}
                 #Activate the GPS to check your position
                 self.activate_gps(widget)
                 lat, lon = self.gps.get_data()
                 #FIXME: GPS stuff
-                feed = self.onceAPI.searchgeo(text, self.page, lat, lon, self.radius)
+                #lat = '43.535000'
+                #lon = '-5.942500'
+                #feed = self.onceAPI.searchgeo(text, self.page, lat, lon, self.radius)
+                #feed = {'entries': {}}
+                glocalresults = self.GLocal.searchgeo(text, self.page, lat, lon, self.radius)
             #If search for location has a text defined
             elif self.location:
-                glocalresults = {}
+                #glocalresults = {}
                 feed = self.onceAPI.searchlocated(text, self.location, self.page)
+                glocalresults = self.GLocal.searchlocated(text, self.location, self.page)
             #Normal search
             else:
                 feed = self.onceAPI.search(text, self.page)
                 #feed = self.onceAPI.searchcity(text, self.page)
-                #glocalresults = self.GLocal.search(text, self.page)
-                glocalresults = {}
+                glocalresults = self.GLocal.search(text, self.page)
+                #glocalresults = {}
 
             #Parse the dictionary feed with the data from 11870
             searchlist = self.parse_results(feed, glocalresults)
@@ -884,6 +1052,7 @@ class MainWindow:
                 #Show the buttons to change the page in the search
                 self.backbtn.show()
                 self.forwbtn.show()
+                self.allmapbtn.show()
         except:
             logger.exception('Searching services')
             winprogind(self.window, 0)
@@ -1037,6 +1206,13 @@ class MainWindow:
         return searchlist
 
     def parse_google_search(self, content):
+        """Add Google Local results to the service details lists
+
+        @type   content: dictionary
+        @param  content: dictionary from a json object
+
+        """
+
         nodata = _('Not available')
 
         for i in content:
@@ -1093,6 +1269,211 @@ class MainWindow:
         for j in medialinks:
             listimg.append('')
 
+    def show_all_in_map(self, widget):
+        """Show a map with all services from a search
+
+        @type   widget: gtk.Widget
+        @param  widget: gtk.Widget that emits the signal
+
+        """
+        win = hildon.StackableWindow()
+        win.set_title(_("All services in map"))
+
+        vbox = gtk.VBox()
+        themap = osmgpsmap.GpsMap(repo_uri = \
+                osmgpsmap.source_get_repo_uri(int(self.mapsource)), \
+                tile_cache = HOME + '/MyDocs/.maps')
+
+        themap.add_layer(osmgpsmap.GpsMapOsd())
+        themap.connect('button_press_event', self.map_clicked)
+
+        vbox.pack_start(themap, True, True, 0)
+
+        hbox = gtk.HBox()
+        button = hildon.Button(fhsize, horbtn)
+        button.connect("clicked", self.zoomin_map, themap)
+        img = gtk.image_new_from_icon_name("pdf_zoomin",
+                                            gtk.ICON_SIZE_LARGE_TOOLBAR)
+        button.set_image(img)
+        hbox.pack_start(button, True, True, 0)
+
+        #button = hildon.Button(fhsize, horbtn)
+        #button.connect("clicked", self.center_map_on_service, lat, lon)
+        #img = gtk.image_new_from_icon_name("sketch_circle",
+        #                                    gtk.ICON_SIZE_LARGE_TOOLBAR)
+        #button.set_image(img)
+        #hbox.pack_start(button, True, True, 0)
+
+        button = hildon.Button(fhsize, horbtn)
+        button.connect("clicked", self.zoomout_map, themap)
+        img = gtk.image_new_from_icon_name("pdf_zoomout",
+                                            gtk.ICON_SIZE_LARGE_TOOLBAR)
+        button.set_image(img)
+        hbox.pack_start(button, True, True, 0)
+
+        button = hildon.Button(fhsize, horbtn)
+        button.connect("clicked", self.show_your_position_on_map, themap)
+        img = gtk.image_new_from_icon_name("gps_location",
+                                            gtk.ICON_SIZE_LARGE_TOOLBAR)
+        button.set_image(img)
+        hbox.pack_start(button, True, True, 0)
+
+        vbox.pack_start(hbox, False, False, 0)
+
+        win.add(vbox)
+        win.show_all()
+
+        pixbuf = gtk.gdk.pixbuf_new_from_file(imgdir + 'place.png')
+
+        del latitudes[:]
+        del longitudes[:]
+
+        for coord in gmlpos:
+            lat, lon = coord.split(' ')
+            if coord != 'Not available':
+                latitudes.append(lat)
+                longitudes.append(lon)
+            else:
+                latitudes.append(False)
+                longitudes.append(False)
+            try:
+                themap.add_image(float(lat), float(lon), pixbuf)
+            except ValueError:
+                pass
+
+        #print latitudes, longitudes
+        #Calculate average point to center the map
+        #TODO: if services are very close, more zoom
+        try:
+            tlat = 0.0
+            for nlat in latitudes:
+                if nlat:
+                    tlat = tlat + float(nlat)
+            tlon = 0.0
+            for nlon in longitudes:
+                if tlon:
+                    tlon = tlon + float(nlon)
+
+            tfalses = latitudes.count(False)
+            tnum = len(latitudes)-tfalses
+            mlat = tlat/tnum
+            mlon = tlon/tnum
+
+            themap.set_mapcenter (float(mlat), float(mlon), 3)
+        except Exception, e:
+            print 'Failed to center map: ', e
+
+    def map_clicked(self, themap, event):
+        """Do actions when clicking in a map.
+        Single click: check if the click is in a service
+        Double click: zoom in the map
+
+        @type   themap: osmgpsmap.GpsMap
+        @param  themap: osmgpsmap.GpsMap that emits the signal
+        @type   event: gtk.gdk.Event
+        @param  event: gtk.gdk.Event that emits the signal
+
+        """
+        import math
+        if event.button == 1:
+            if event.type == gtk.gdk._2BUTTON_PRESS:
+                themap.set_zoom(themap.props.zoom + 3)
+            elif event.type == gtk.gdk.BUTTON_PRESS:
+                rlat, rlon = themap.get_co_ordinates(event.x, event.y)
+                #print math.degrees(rlat)
+                #print math.degrees(rlon)
+                self.check_if_tap_service(math.degrees(rlat), math.degrees(rlon), themap)
+
+    def check_if_tap_service(self, taplat, taplon, themap):
+        """Check if a click in the map is in a service
+
+        @type   taplat: string
+        @param  taplat: latitude in the map clicked
+        @type   taplon: string
+        @param  taplon: longitude in the map clicked
+        @type   themap: osmgpsmap.GpsMap
+        @param  themap: osmgpsmap.GpsMap that emits the signal
+
+        """
+
+        #print longitudes
+        #print latitudes
+
+        zoom = themap.props.zoom
+        #print 'zoom; ', zoom
+
+        #Different precision for different map zoom
+        if zoom in range(1,6):
+            decim = "%.f"
+        elif zoom in range(6, 10):
+            decim = "%.1f"
+        elif zoom in range(10, 13):
+            decim = "%.2f"
+        elif zoom in range(13, 17):
+            decim = "%.3f"
+        elif zoom in range(17, 19):
+            decim = "%.4f"
+
+        taplat = decim % float(taplat)
+        taplon = decim % float(taplon)
+
+        #print taplat
+        #print taplon
+
+        for i in range(len(latitudes)):
+            ni = decim % float(latitudes[i])
+            #print 'ni: ', ni
+            if ni == taplat:
+                nj = decim % float(longitudes[i])
+                #print 'nj: ', nj
+                if nj == taplon:
+                    #print 'a service selected'
+                    self.show_service_info_dialog(i)
+                    return
+                break
+
+    def show_service_info_dialog(self, index):
+        """Show a dialog with information about a service
+
+        @type   index: number
+        @param  index: Index of the service in the results list
+
+        """
+        dlg = gtk.Dialog(title=_('Show service information'), parent=None, flags=0)
+        dlg.set_has_separator(False)
+
+        label = gtk.Label()
+        label.set_alignment(0.00, 0.5)
+        label.set_markup('%s' % (cgi.escape(titles[index])))
+        label.set_line_wrap(gtk.WRAP_WORD)
+        dlg.vbox.pack_start(label, True, True, 0)
+
+
+        label = gtk.Label()
+        label.set_alignment(0.00, 0.5)
+        label.set_markup('%s' % (cgi.escape(addresses[index])))
+        label.set_line_wrap(gtk.WRAP_WORD)
+        dlg.vbox.pack_start(label, True, True, 0)
+
+        label = gtk.Label()
+        label.set_alignment(0.00, 0.5)
+        label.set_markup('%s - %s - %s' % (cgi.escape(localities[index]),
+                                cgi.escape(areas[index]),
+                                cgi.escape(countries[index])))
+        label.set_line_wrap(gtk.WRAP_WORD)
+        dlg.vbox.pack_start(label, True, True, 0)
+
+
+        if phones[index] != _('Not available'):
+            button = hildon.Button(fhsize, verbtn)
+            button.set_title(_('Phone'))
+            button.set_value(phones[index])
+            button.connect("clicked", self.make_phonecall, phones[index])
+            dlg.vbox.pack_start(button, False, False, 0)
+
+        dlg.show_all()
+        result = dlg.run()
+        dlg.destroy()
 
     def show_info_banner(self, widget, msg):
         """Show a information banner
@@ -1183,6 +1564,16 @@ class MainWindow:
         dlg.destroy()
 
     def create_selector(self, data):
+        """Create a hildon.TouchSelector with some data in.
+
+        @type   data: list
+        @param  data: data to be added to the touchselector
+
+        @rtype: hildon.TouchSelector
+        @returns: a hildon.TouchSelector to be added to a parent widget
+
+        """
+
         selector = hildon.TouchSelector(text=True)
 
         for i in range(len(data)):
@@ -1191,17 +1582,25 @@ class MainWindow:
         return selector
 
     def make_phonecall(self, widget, phonenumber):
+        """Make a phone call with the OS phone application
+
+        @type   widget: gtk.Widget
+        @param  widget: gtk.Widget that emits the signal
+        @type   phonenumber: string
+        @param  phonenumber: Phone number to be called
+
+        """
+
         import subprocess, shlex
         phonenumber = phonenumber.replace(' ', '')
         call = 'dbus-send --system --dest=com.nokia.csd.Call --type=method_call --print-reply /com/nokia/csd/call com.nokia.csd.Call.CreateWith string:%s uint32:0' % phonenumber
-        print call
+        #print call
 
         args = call.split(' ')
 
-        print args
+        #print args
         p = subprocess.Popen(args)
 
-
 #TODO: make this in the same class, easier to get data??
 
 class GPS:
